

//	(C) 2017 R-MIK Sawomir Skrzyski
//	slawomir.skrzynski@ep.com.pl


//#define	USART0_BITS		8			// 5..9 (FTDI 7..9)
//#define	USART0_PARITY	'n'			// 'n', 'e', 'o'
//#define	USART0_STOP		1			// 1, 2
//#define	USART0_IRQRX
//#define	USART0_RXBUF	16			// ! Potega dwjki
//#define	USART0_IRQTX
//#define	USART0_TXBUF	16			// ! Potega dwjki


#include <rmik/usart0.h>


//=======================================================================//
//				Obsuga RS (UART / USART)							//
//------------------------------------------------------------------//
void Usart0_Init()
{
byte ursel=0;


	#ifdef __AVR_ATmega162__
	  ursel = (1<<URSEL0);
	  UCSR0C &= ~(1<<URSEL0);
	#endif
	UBRR0H = UBRRH_VALUE;				// Set baud (wybrane przez BAUD)
	UBRR0L = UBRRL_VALUE;

	#if USART0_BITS==5
	  UCSR0C = (0<<UCSZ00) | ursel;		// Set frame format: 5n1
	#elif USART0_BITS==6
	  UCSR0C = (1<<UCSZ00) | ursel;		// Set frame format: 6n1
	#elif USART0_BITS==7
	  UCSR0C = (2<<UCSZ00) | ursel;		// Set frame format: 7n1
	#elif USART0_BITS==8 || USART0_BITS==9
	  UCSR0C = (3<<UCSZ00) | ursel;		// Set frame format: 8n1 lub 9n1
	#else
	  #error "Zla liczba bitow"
	#endif
	#if USART0_STOP==1
	  //UCSR0C |= (0<<USBS0) | ursel; 	// xx1
	#elif USART0_STOP==2
	  UCSR0C |= (1<<USBS0) | ursel; 	// xx2
	#else
	  #error "Zla liczba bitow stopu"
	#endif
	#if USART0_PARITY=='n'
	  //UCSR0C |=(0<<UPM0);				// xNx
	#elif USART0_PARITY=='e'
	  UCSR0C |=(2<<UPM0);				// xNx
	#elif USART0_PARITY=='o'
	  UCSR0C |=(3<<UPM0);				// xNx
	#else
	  #error "Zla parzystosc"
	#endif
	UCSR0B = (1<<RXEN0) | (1<<TXEN0)	// Enable receiver and transmitter
			#if USART0_BITS==9
			  | (1<<UCSZ02)				// Set frame format: 9n1 (UCSZ=7)
			#endif
			#ifdef USART0_IRQRX
			  | (1<<RXCIE0)				// Waczenie przerwa odborczych
			#endif
			#ifdef USART0_IRQTX
			  | (1<<TXCIE0)
			#endif
			;

	#if USE_2X
	 UCSR0A |= _BV(U2X0);
	#endif
}


//------------------------------------------------------------------//
//				Sprawdzenie czy jest znak w RS						//
//------------------------------------------------------------------//
char Usart0_Test()
{
	#ifdef USART0_IRQRX
	  if ( Usart0_PtrRxR != Usart0_PtrRxW ) return( TRUE );
	#else
	  if ( (UCSR0A & (1<<RXC0)) ){ return( TRUE ); }		// Jeli odernao znak to wyjd z funkcji
	#endif

	return( FALSE );									// Nie ma znaku
}


//------------------------------------------------------------------//
//				Czekanie na znak z RS 								//
//------------------------------------------------------------------//
word Usart0_Receive()
{
char znak;

	wdt_reset();
	if ( (UCSR0A & (1<<RXC0)) )									// Jeli odernao znak to odczytaj go i wyjd z funkcji
		{
		if ( (UCSR0A & (1<UPE0)) ){ return( PARITYERR ); }		// Sprawdzenie parzystoci
		if ( (UCSR0A & (1<FE0)) ) { return( FRAMEERR ); }		// Sprawdzenie bdu ramki (np zy znak stopu)
		if ( (UCSR0A & (1<DOR0)) ){ return( OVERRUNERR ); }		// Sprawdzenie przepenienia bufora
		znak = UDR0;
		if ( UCSR0B & RXB80 ) znak |= 0x100;
		return( znak );
		}

	return ( NONE );			// Min timeout, wyjd z bdem
}


//------------------------------------------------------------------//
//				Wysanie znaku po RS								//
//------------------------------------------------------------------//
void Usart0_Transmit( word data )
{
	/* Wait for empty transmit buffer */
	while ( !( UCSR0A & (1<<UDRE0)) ){ ; }

	#if USART0_BITS==9
	  if ( data & 0x100 ) { UCSR0B |= (1<<TXB80); } else { UCSR0B &= ~(1<<TXB80); }
	#endif
	/* Put data into buffer, sends the data */
	UDR0 = data;
}


//------------------------------------------------------------------//
//				Czyszczenie FIFO UARTA								//
//------------------------------------------------------------------//
void Usart0_Flush()
{
	#ifdef USART0_IRQRX
	  Usart0_PtrRxR = Usart0_PtrRxW = 0;
	#else
	  byte dummy;
	  while ( UCSR0A & (1<<RXC0) ) dummy = UDR0;
	#endif
}


//----------------------------------------------------------------------//
///* USART0, Rx Complete */
///* USART0 RX complete */
//#define USART0_RX_vect			_VECTOR(28)
//#define SIG_Usart0_RECV			_VECTOR(28)

///* USART0 Data register Empty */
///* USART0 Data Register Empty */
//#define USART_UDRE_vect			_VECTOR(29)
//#define SIG_USART_DATA			_VECTOR(29)
//----------------------------------------------------------------------//
#ifdef USART0_IRQRX
//#define	SIG_USART_RECV		USART0_RX_vect
//#define	SIG_USART0_RECV		USART0_RX_vect
SIGNAL(USART0_RX_vect)			// Musi byc ISR lub SIGNAL a nie INTERUPT
{
word bit8=0;

#warning "zaimplementowac sprawdzanie FIFO"
// petla do...while( UCSR0A & (1<<RXC0) )
// to samo dla usart 1


	#if USART0_BITS==9
	  if ( UCSR0B & (1<<RXB80) ) bit8 = 0x100;
	#endif
	Usart0_RxBuf[Usart0_PtrRxW++] = UDR0 | bit8;	// wpisz znak do bufora i zwiksz dugo
	Usart0_PtrRxW &= USART0_RXBUF-1;					// Ograniczenie wieloci bufora (max 255 bajtw)
	if ( Usart0_PtrRxW == Usart0_PtrRxR )			// Jeli wskanik zapisu "dogoni" wskanik odczytu
		{											// to przepeniono bufor RS-a
//		PrintString_P( PSTR(ERR_BUF_USB_TXT""CRLF) );
		}
}
#endif


//----------------------------------------------------------------------//
word Usart0_Get()
{
	#ifdef USART0_IRQRX
	  if ( Usart0_PtrRxR == Usart0_PtrRxW ) return( NONE );
	  return( Usart0_RxBuf[ Usart0_PtrRxR ++ ] );
	#else
	  return( Usart0_Receive() );
	#endif
}


//----------------------------------------------------------------------//
byte Usart0_Put( word c )
{
	#ifdef USART0_IRQTX
//todo: jeli jest miejsce
	if ( Usart0_PtrTxR )

	  Usart0_TxBuf[ Usart0_PtrTxW++ ] = c;
	  Usart0_PtrTxW &= DEF_TXBUF0-1
//todo: zrobi na IRQ napenianie bufora nadawczego, jak nie mona to zwr FALSE
// Jak rozwiza problem, e znak jest pierwszy?
//	Moe na zasadzie znanikw?

	#else
	  Usart0_Transmit( c ); return( TRUE );
	#endif
}


//----------------------------------------------------------------------//
#ifdef USART0_IRQTX
SIGNAL(USART_UDRE_vect)			// Musi byc ISR lub SIGNAL a nie INTERUPT
{
	#if USART0_BITS==9
	  if ( Usart0_TxBuf[ Usart0_PtrTxR] & 0x100 ) UCRS0B |= (1<<TXRB80); else UCRS0B &= ~(1<<TXRB80);
	#endif
	UDR0 = Usart0_TxBuf[ Usart0_PtrTxR++ ];
#error	ograniczenie zakresu
}
#endif


//----------------------------------------------------------------------//
