

//	(C) 2017 R-MIK Sawomir Skrzyski
//	slawomir.skrzynski@ep.com.pl


//#define	USART1_BITS		8			// 5..9 (FTDI 7..9)
//#define	USART1_PARITY	'n'			// 'n', 'e', 'o'
//#define	USART1_STOP		1			// 1, 2
//#define	USART1_IRQRX
//#define	USART1_RXBUF	16			// ! Potega dwjki
//#define	USART1_IRQTX
//#define	USART1_TXBUF	16			// ! Potega dwjki


#include <rmik/usart1.h>


//=======================================================================//
//				Obsuga RS (UART / USART)							//
//------------------------------------------------------------------//
void Usart1_Init()
{
byte ursel=0;


	#ifdef __AVR_ATmega162__
	  ursel = (1<<URSEL1);
	  UCSR1C &= ~(1<<URSEL1);
	#endif
	UBRR1H = UBRRH_VALUE1;				// Set baud (wybrane przez BAUD)
	UBRR1L = UBRRL_VALUE1;

	#if USART1_BITS==5
	  UCSR1C = (0<<UCSZ10) | ursel;		// Set frame format: 5n1
	#elif USART1_BITS==6
	  UCSR1C = (1<<UCSZ10) | ursel;		// Set frame format: 6n1
	#elif USART1_BITS==7
	  UCSR1C = (2<<UCSZ10) | ursel;		// Set frame format: 7n1
	#elif USART1_BITS==8 || USART1_BITS==9
	  UCSR1C = (3<<UCSZ00) | ursel;		// Set frame format: 8n1 lub 9n1
	#else
	  #error "Zla liczba bitow"
	#endif
	#if USART1_STOP==1
	  //UCSR1C |= (0<<USBS1) | ursel; 	// xx1
	#elif USART1_STOP==2
	  UCSR1C |= (1<<USBS1) | ursel; 	// xx2
	#else
	  #error "Zla liczba bitow stopu"
	#endif
	#if USART1_PARITY=='n'
	  //UCSR1C |=(0<<UPM1);				// xNx
	#elif USART1_PARITY=='e'
	  UCSR1C |=(2<<UPM1);				// xNx
	#elif USART1_PARITY=='o'
	  UCSR1C |=(3<<UPM1);				// xNx
	#else
	  #error "Zla parzystosc"
	#endif
	UCSR1B = (1<<RXEN1) | (1<<TXEN1)		// Enable receiver and transmitter
			#if USART1_BITS==9
			  | (1<<UCSZ12)					// Set frame format: 9n1 (UCSZ=7)
			#endif
			#ifdef USART1_IRQRX
			  | (1<<RXCIE1)					// Waczenie przerwa odborczych
			#endif
			#ifdef USART1_IRQTX
			  | (1<<TXCIE1)
			#endif
			;

	#if USE1_2X
	 UCSR1A |= _BV(U2X1);
	#endif
}


//------------------------------------------------------------------//
//				Sprawdzenie czy jest znak w RS						//
//------------------------------------------------------------------//
char Usart1_Test()
{
	#ifdef USART1_IRQRX
	  if ( Usart1_PtrRxR != Usart1_PtrRxW ) return( TRUE );
	#else
	  if ( (UCSR1A & (1<<RXC1)) ){ return( TRUE ); }		// Jeli odernao znak to wyjd z funkcji
	#endif

	return( FALSE );									// Nie ma znaku
}


//------------------------------------------------------------------//
//				Czekanie na znak z RS 								//
//------------------------------------------------------------------//
word Usart1_Receive()
{
char znak;

	wdt_reset();
	if ( (UCSR1A & (1<<RXC1)) )									// Jeli odernao znak to odczytaj go i wyjd z funkcji
		{
		if ( (UCSR1A & (1<UPE1)) ){ return( PARITYERR ); }		// Sprawdzenie parzystoci
		if ( (UCSR1A & (1<FE1)) ) { return( FRAMEERR ); }		// Sprawdzenie bdu ramki (np zy znak stopu)
		if ( (UCSR1A & (1<DOR1)) ){ return( OVERRUNERR ); }		// Sprawdzenie przepenienia bufora
		znak = UDR1;
		if ( UCSR1B & RXB81 ) znak |= 0x100;
		return( znak );
		}

	return ( NONE );			// Min timeout, wyjd z bdem
}


//------------------------------------------------------------------//
//				Wysanie znaku po RS								//
//------------------------------------------------------------------//
void Usart1_Transmit( word data )
{
	/* Wait for empty transmit buffer */
	while ( !( UCSR1A & (1<<UDRE1)) ){ ; }

	#ifdef USART1_9BIT
	  if ( data & 0x100 ) { UCSR1B |= (1<<TXB81); } else { UCSR1B &= ~(1<<TXB81); }
	#endif
	/* Put data into buffer, sends the data */
	UDR1 = data;
}


//------------------------------------------------------------------//
//				Czyszczenie FIFO UARTA								//
//------------------------------------------------------------------//
void Usart1_Flush()
{
	#ifdef USART1_IRQRX
	  Usart1_PtrRxR = Usart1_PtrRxW = 0;
	#else
	  byte dummy;
	  while ( UCSR1A & (1<<RXC1) ) dummy = UDR1;
	#endif
}


//----------------------------------------------------------------------//
///* USART1, Rx Complete */
///* USART1 RX complete */
//#define USART1_RX_vect			_VECTOR(28)
//#define SIG_Usart1_RECV			_VECTOR(28)

///* USART1 Data register Empty */
///* USART1 Data Register Empty */
//#define USART_UDRE_vect			_VECTOR(29)
//#define SIG_USART_DATA			_VECTOR(29)
//----------------------------------------------------------------------//
#ifdef USART1_IRQRX
//#define	SIG_USART1_RECV		USART1_RX_vect
SIGNAL(USART1_RX_vect)			// Musi byc ISR lub SIGNAL a nie INTERUPT
{
word bit8=0;


	#ifdef USART1_9BIT
	  if ( UCSR1B & (1<<RXB81) ) bit8 = 0x100;
	#endif
	Usart1_RxBuf[Usart1_PtrRxW++] = UDR1 | bit8;	// wpisz znak do bufora i zwiksz dugo
	Usart1_PtrRxW &= USART1_RXBUF-1;					// Ograniczenie wieloci bufora (max 255 bajtw)
	if ( Usart1_PtrRxW == Usart1_PtrRxR )			// Jeli wskanik zapisu "dogoni" wskanik odczytu
		{											// to przepeniono bufor RS-a
//		PrintString_P( PSTR(ERR_BUF_USB_TXT""CRLF) );
		}
}
#endif


//----------------------------------------------------------------------//
word Usart1_Get()
{
	#ifdef USART1_IRQRX
	  if ( Usart1_PtrRxR == Usart1_PtrRxW ) return( NONE );
	  return( Usart1_RxBuf[ Usart1_PtrRxR ++ ] );
	#else
	  return( Usart1_Receive() );
	#endif
}


//----------------------------------------------------------------------//
byte Usart1_Put( word c )
{
	#ifdef USART1_IRQTX
//todo: jeli jest miejsce
	if ( Usart1_PtrTxR )

	  Usart1_TxBuf[ Usart1_PtrTxW++ ] = c;
	  Usart1_PtrTxW &= DEF_TXBUF1-1
//todo: zrobi na IRQ napenianie bufora nadawczego, jak nie mona to zwr FALSE
// Jak rozwiza problem, e znak jest pierwszy?
//	Moe na zasadzie znanikw?

	#else
	  Usart1_Transmit( c ); return( TRUE );
	#endif
}


//----------------------------------------------------------------------//
#ifdef USART1_IRQTX
SIGNAL(USART1_UDRE_vect)			// Musi byc ISR lub SIGNAL a nie INTERUPT
{
	#ifdef USART1_9BIT
	  if ( Usart1_TxBuf[ Usart1_PtrTxR] & 0x100 ) UCRS1B |= (1<<TXRB81); else UCRS1B &= ~(1<<TXRB81);
	#endif
	UDR1 = Usart1_TxBuf[ Usart1_PtrTxR++ ];
#error	ograniczenie zakresu
}
#endif


//----------------------------------------------------------------------//
